package OpeniT::Contrib::Config;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %_KnownExtensions);

require Exporter;

@ISA = qw(Exporter);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(

);
$VERSION = '0.01';


# Preloaded methods go here.
### Common methods for all file types:
sub new {
    my $class = shift;
    my $file  = shift;

    my $self = { };

    bless $self, $class;

    if( !defined $file ) {
	$file = "/etc/opt/openit/openit.cfg";
    }
    if( -r $file ) {
	$self->read($file);
    }

    return $self;
}

sub clear {
    my $self = shift;
    foreach my $key ( keys %$self ) {
	delete $self->{$key};
    }

    return $self;
}

sub read {
    my $self = shift;
    my $config_file = shift;
    if( ! defined $config_file ) {
	$config_file = "/etc/opt/openit/openit.cfg";
    }

    my %unresolved;

    local *CFG;
    open( CFG, "<$config_file" ) or die "OpeniT::Contrib::Config::read(): Could not read configuration file '$config_file'\n";

    # Read the file - directives referencing others are dealt with later
    while( <CFG> ) {
	s/#.*$//;         # Remove comments
	next if /^\s*$/;  # Skip blank lines

	# Remove leading and trailing whitespace
	s/^\s+//;
	s/\s+$//;

	my( $key, $value ) = split(/\s+/, $_, 2);

	if( $value =~ /\$/ ) {   # Does it contains one or more references?
	    my $give_up = 0;
	    # Resolve backward references:
	    while( $value =~ /\$([\w\d_+\-]+)/ && !$give_up) {
		my $ref = $1;
		if( exists $self->{$ref} ) {
		    my $repl=$self->{$ref};
		    $value =~ s/\$$ref/$repl/g;
		}
		else {
		    $give_up = 1;  # Forward reference - give up.
		}
	    }
	    if( $give_up ) {
		$self->{_unresolved}->{$key} = $value;
		++$self->{_unresolved_count};
	    }
	    else {
		$self->{$key} = $value;
	    }
	}
	else {   # Contains a literal value
	    $self->{$key} = $value;
	}
    }
    close CFG;

    # Now resolve references
#    while( my( $ukey, $uexpr ) = each( %{$self->{_unresolved}} ) ) {
#	my $prev_ref;
#	while( $uexpr =~ /\$([\w\d_+\-]+)/ && $1 ne $prev_ref ) {
#	    my $ref = $1;
#	    my $value = $self->_resolve_symbol(\%param, $ref);

    return $self;
}

# "private"/internal subroutines, NOT FINISHED:
sub _resolve_symbol {

    my $self = shift;
    my $symbol = shift;
    my $trace = shift;
    if( !defined $trace ) {
	$trace = ":";
    }
    if( $trace =~ /:$symbol(:|$)/ ) {
	die "OpeniT::Contrib::Config::_resolve_symbol($symbol): Circular reference ($trace:$symbol), check configuration file\n";
    }

    if( exists $self->{$symbol} ) {
	return $self->{$symbol};
    }
    else {


#	$uexpr =~ s/\$$ref/$value/g;
    }
}

sub unresolved_count {
    my $self = shift;
    if( exists $self->{_unresolved_count} ) {
	return $self->{_unresolved_count};
    }
    else {
	return 0;
    }
}

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

OpeniT::Contrib::Config - Perl extension for reading Open iT Resources'
main configuration.

=head1 SYNOPSIS

  use OpeniT::Contrib::Config;
  my $config = new OpeniT::Contrib::Config;
  $config->read("another/config/file.cfg");
  my $setting = $config->{DATA_DIR};

=head1 DESCRIPTION

This is a support module for perl scripts which need access to Open iT
Resources' main configuration, for finding the directories or reading
other settings. It's a simple, non-validating parser (it will gladly
accept just about anything), and it does not yet handle forward
references (they are illegal in Open iT's main configuration anyway).

It provides these methods:

  new
          Create a config object. You may supply a filename, in which
	  case the it will call read(filename) for you (if it exists).
	  Without any parameters, it will call read() on the default
	  config file /etc/opt/openit/openit.cfg

  read
          Read a configuration file, and add the directives found to
          the object. If directives with the same name is present in
          two files, the most recently read will overwrite the old
          value. If no argument is supplied, it will default to
          /etc/opt/openit/openit.cfg

  clear
          Forget everything

  unresolved_count
          Returns the number of unresolved references. A non-zero
          value indicates that a forward reference was encountered
          (perhaps because two or more interdependent files where
          read in the wrong order), or that one or more references
          were made to non-existant directives (or you forgot to
          read a file).


=head1 AUTHOR



Ketil Kristiansen, Open Systems Computing A/S, ketil-k@osc.no

=head1 SEE ALSO

perl(1).

=cut
