#!/etc/opt/openit/perl
#
# $Id: db_clean_structure.pl,v 1.1.2.14 2012/06/04 11:37:16 kyri Exp $
#
# (C) Copyright 2012 by Open iT inc., All rights reserved.
#
# This software may be used or copied only with the written permission by
# Open Systems Consultans a.s or in accordance with the terms and conditions
# stipulated in the agreement under which this software has been supplied.
#
# This file is part of Open iT.
#
# NAME:
#   db_clean_structure
# AUTHOR:
#   Kjetil Yri
# CREATED:
#   Thu Apr 12 10:05:06 MET 2012
# DESCRIPTION:
#   Test consistency of Reporting Server data base
#
###############################################################################

BEGIN {
    push @INC, "../lib";
    push @INC, "/opt/openit/contrib/lib";
}


use strict;

use OpeniT::Contrib::Config;
use OpeniT::Contrib::File;
use OpeniT::Contrib::DBCleanStructure;

use DirHandle;
use Getopt::Long;
use Fcntl qw( :DEFAULT :flock );

my $opt_info;
my $opt_ignore_lock;
my $opt_help;
my $opt_debug;
my $opt_verbose;

unless ( GetOptions(
		    "debug"       => \ $opt_debug,
		    "verbose"     => \ $opt_verbose,
		    "info"        => \ $opt_info,
                    "ignore_lock" => \ $opt_ignore_lock, # For test purposes
		    "help"        => \ $opt_help,

 )) {
    &print_usage;
}

if ($opt_help) {
    &print_usage;
}

my $config = new OpeniT::Contrib::Config;
$config->read( "/etc/opt/openit/openit.cfg" );

my $lock_file = $config->{TEMP_DIR} .
  &OpeniT::Contrib::File::file_separator() . "database_update.lock";
open( LOCK_HANDLE, ">$lock_file" ) or die "Could not open $lock_file: $!\n";
if( ! $opt_ignore_lock && ! flock( LOCK_HANDLE, LOCK_EX | LOCK_NB ) ) {
    warn "WARNING: Another database_update process is running. Exiting!\n";
    exit( 0 );
}

if( $opt_debug ) {
    my $debug_filename = "/tmp/db_clean_structure.log";
    OpeniT::Contrib::Debug::set_debug_filename( $debug_filename );
}

my %datatypes;
foreach my $arg( @ARGV ) {
    $datatypes{ $arg } = 1;
}

my $filesep = &OpeniT::Contrib::File::file_separator();

if( $opt_info ) {
    OpeniT::Contrib::DBCleanStructure::set_info( 1 );
}

my $db_path = $config->{DATA_DIR};

print "Processing db structure cleanup in $db_path\n";

my @datatype_paths = ();

my $dir_handle = new DirHandle $db_path;

if (! defined($dir_handle)) {
    die "Couldn't read dir $db_path: $!";
}

my $dir_entry;
while (defined($dir_entry = $dir_handle->read)) {
    next if ($dir_entry eq "." || $dir_entry eq "..");
    next if ($dir_entry !~ m/^(\d+)$/ && $dir_entry !~ /^(olap.*)$/ );
    my $dt = $1;
    if( scalar keys %datatypes && ! defined $datatypes{ $dir_entry } ) {
	next;
    }
    print "Cleaning up for data type $dt\n";
    OpeniT::Contrib::DBCleanStructure::process_data( $dt );
    print "Done cleanup for data type $dt\n";
}
undef $dir_handle;

if( ! flock( LOCK_HANDLE, LOCK_UN ) ) {
    warn "Unable to release $lock_file: $!\n";
}


sub print_usage {
    die "Usage:\n" . "$0 [options] [<dt-1> [<dt-2> [...]]]\n" .
      "<dt...> is datatype(s)\n" .
	"  Options:\n" . "  --help\n" . "  --info\n" .
	  "  --debug\n" . "  --verbose\n";
}
